module Color where

import Language.Fay.Prelude

data Color = RGBA Double Double Double Double

white, black :: Color
white = RGBA 1 1 1 1
black = RGBA 0 0 0 1

-- Primary and secondary colors
red, green, blue, cyan, magenta, yellow :: Color
red        = RGBA 1 0 0 1
green      = RGBA 0 1 0 1
blue       = RGBA 0 0 1 1
yellow     = RGBA 1 1 0 1
cyan       = RGBA 0 1 1 1
magenta    = RGBA 1 0 1 1

-- Tertiary colors
orange, rose, chartreuse, aquamarine, violet, azure :: Color
orange     = RGBA 1.0 0.5 0.0 1
rose       = RGBA 1.0 0.0 0.5 1
chartreuse = RGBA 0.5 1.0 0.0 1
aquamarine = RGBA 0.0 1.0 0.5 1
violet     = RGBA 0.5 0.0 1.0 1
azure      = RGBA 0.0 0.5 1.0 1

-- Other common colors and color names
brown  = RGBA 0.7 0.4 0.3 1
purple = violet
pink   = light rose

light :: Color -> Color
light (RGBA r g b a) = RGBA
    (min 1 (r + 0.4))
    (min 1 (g + 0.4))
    (min 1 (b + 0.4))
    a

dark :: Color -> Color
dark (RGBA r g b a) = RGBA
    (max 0 (r - 0.4))
    (max 0 (g - 0.4))
    (max 0 (b - 0.4))
    a

translucent :: Color -> Color
translucent (RGBA r g b a) = RGBA r g b (0.5 * a)

gray, grey :: Double -> Color
gray = grey
grey k = RGBA k k k 1
