{-
  A very simple interactive game, which demonstrates the use of events.  Games
  are composed of pieces called initial, step, event, and draw, with types:

  initial :: StdGen -> World
  step :: Double -> World -> World
  event :: GameEvent -> World -> World
  draw :: World -> Picture

  World is a type defined by you.  Use initial to describe the world at the
  start of the game.  Use step to explain how the world changes as time goes
  by.  Use event to explain how the world changes when the user does something.
  Use draw to explain what the world looks like.
-}

import CodeWorld

main = playInCanvas initial step event draw

data World = World Point Point

distance (x1,y1) (x2,y2) = sqrt ((x2 - x1)^2 + (y2 - y1)^2)

initial g = World (100,100) (0, 0)

step dt (World (cx, cy) (mx, my))
  | d1 >= d2  = World (mx, my) (mx, my)
  | otherwise = World (cx + d1 / d2 * (mx - cx), cy + d1 / d2 * (my - cy))
                      (mx, my)
  where d1 = dt * 100
        d2 = distance (cx, cy) (mx, my)

event (MouseMoveEvent p) (World cat _) = World cat p
event _                  w             = w

draw (World (cx, cy) (mx, my)) = pictures [
    translate cx cy cat,
    translate mx my mouse
    ]

cat   = color brown  (solidCircle 20)
mouse = color purple (solidCircle 10)
