{-
  This is a simulation of juggling patterns.  Simulations are composed of
  pieces called initial, step, and draw, with types:

  initial :: StdGen -> World
  step :: Double -> World -> World
  draw :: World -> Picture

  World is a type defined by you.  (In this case, it's an unnamed tuple type.)
  Use initial to describe the world at the start of the simulation.  Use step
  to explain how the world changes as time goes by.  Use draw to explain what
  the world looks like.
-}

import CodeWorld

main = simulateInCanvas initial step draw

thePattern = [4,5,3]

initial g = (-1, 0.0, [], cycle thePattern)

step dt (hand, time, balls, pattern) = (newhand, newtime, newballs, newpattern)
    where divided    = properFraction (time + dt)
          throw      = fst divided
          newtime    = snd divided
          newhand    = if throw == 1 then -hand else hand
          thrown     = if throw == 1 then [ (hand, newtime, head pattern) ]
                                     else []
          newpattern = if throw == 1 then tail pattern else pattern
          newballs   = [ (bhand, btime + dt, height)
                       | (bhand, btime, height) <- balls,
                         btime + dt < fromIntegral height ]
                       ++ thrown

draw (hand, time, balls, pattern) = pictures [
    juggler,
    pictures [ ball b | b <- balls ]
    ]

ball (bhand, btime, height) = translate (50*x) (50*y) (solidCircle 10)
    where t = 1 - 2 * (btime / fromIntegral height)
          x = if even height then bhand else bhand * t
          y = if height < 3 then 0 else fromIntegral (height - 1) * (1 - t^2)

juggler = pictures [
    line [(-50, 0), (0, 25), (50, 0)],
    line [(-30, -100), (0, -50), (30, -100)],
    line [( 0, 25), (0, -50)],
    translate 0 50 (circle 25)
    ]

